const server = require('../../server');
const { render, status } = server.reply;

// Test runner:
const run = require('server/test/run');

describe('express', () => {
  it('is defined', () => {
    server(parseInt(1000 + Math.random() * 10000)).then(ctx => {
      expect(ctx.app).toBeDefined();
      ctx.close();
    });
  });

  it('accepts the options', async () => {

    const options = {
      'case sensitive routing': true,
      'etag': 'strong',
      'jsonp callback name': 'abc',
      'subdomain offset': 1,
      'trust proxy': true,
      'view cache': true,
      'x-powered-by': false
    };

    const res = await run({ express: options }, ctx => {
      for (let key in options) {
        expect(ctx.app.get(key)).toBe(options[key]);
      }
      return status(200);
    }).get('/');
    expect(res.status).toBe(200);
    expect(res.body).toBe('');
  });

  it('ignores the view engine (use .engine instead)', async () => {
    const res = await run({ express: { 'view engine': 'abc' } }, ctx => {
      expect(ctx.app.get('env')).toBe('test');
      expect(ctx.app.get('view engine')).toBe('pug');
      return status(200);
    }).get('/');
    expect(res.status).toBe(200);
    expect(res.body).toBe('');
  });

  it.only('uses an engine', async () => {
    const engine = {
      bla: function (file, options, callback) {
        callback(null, 'Hello world');
      }
    };
    const res = await run({ engine },
      () => render('index.bla')
    ).get('/');
    expect(res.data).toBe('Hello world');
    expect(res.status).toBe(200);
  });
});
